EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Assets_DeleteItem',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Marks the specified asset as deleted and optionally creates an entry in the old versions table.'
GO

IF OBJECT_ID('[Assets_DeleteItem]') IS NOT NULL DROP PROCEDURE [Assets_DeleteItem]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Assets_DeleteItem]
(
	@AssetItem_Id INT,
	@AddToHistory_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

	UPDATE [AssetItems]
	   SET [Deleted_Date] = GETUTCDATE()
	 WHERE [AssetItem_Id] = @AssetItem_Id


	IF @AddToHistory_Indicator = 'Y'
	BEGIN

		INSERT INTO [AssetItemOldVersions]
		(
			[AssetItem_Id],
			[SHA256Hash_Bytes],
			[Deleted_Date],
			[DeletedBy_User_Name],
			[Modified_Date],
			[ModifiedBy_User_Name],
			[AssetItem_Size]
		)
		SELECT @AssetItem_Id,
		       [SHA256Hash_Bytes],
			   [Deleted_Date],
			   dbo.Session_GetUserName(),
			   COALESCE([Modified_Date], [Created_Date]),
			   COALESCE([ModifiedBy_User_Name], [CreatedBy_User_Name]),
			   [AssetItem_Size]
		  FROM [AssetItems]
		 WHERE [AssetItem_Id] = @AssetItem_Id

	END

	DECLARE @Feed_Id INT = (SELECT [Feed_Id] FROM [AssetItems] WHERE [AssetItem_Id] = @AssetItem_Id)

	UPDATE [Feeds]
	   SET [FeedState_Number] = COALESCE([FeedState_Number], 0) + 1
	 WHERE [Feed_Id] = @Feed_Id

END
GO

GRANT EXECUTE ON [Assets_DeleteItem] TO [ProGetUser_Role]