EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Assets_SearchItems',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'AssetItems_SearchResults',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Searches the items in an asset directory based on a search string.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[Assets_SearchItems]') IS NOT NULL DROP PROCEDURE [Assets_SearchItems]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Assets_SearchItems]
(
    @Feed_Id INT,
    @Search_Text NVARCHAR(200),
    @BasePath_Text NVARCHAR(MAX) = NULL,
	@Row_Count INT = 100,
    @IncludeDeleted_Indicator YNINDICATOR = 'N'
)
AS
BEGIN

	-- consider . and / to be spaces
    SET @Search_Text = LOWER(REPLACE(REPLACE(@Search_Text,'.', ' '),'/', ' '))
    SET @BasePath_Text = NULLIF(@BasePath_Text,'')

	-- find the assetId of the base item
    DECLARE @BaseAssetItem_Id INT
    IF @BasePath_Text IS NOT NULL
        EXEC [Assets_FindFromPath] 
            @Feed_Id = @Feed_Id, 
            @AssetPath_Text = @BasePath_Text, 
            @IncludeDeleted_Indicator = @IncludeDeleted_Indicator, 
            @AllowCaseInsensitiveMatch_Indicator = 'Y',
            @AssetItem_Id = @BaseAssetItem_Id OUT, 
            @CanonicalAssetPath_Text = NULL, 
            @Inherited_CacheConfiguration_XML = NULL

	-- basic word match with items under the base
    ;WITH AI_UnderBase AS (
      SELECT [AssetItem_Id] FROM [AssetItems] WHERE [Parent_AssetItem_Id] = @BaseAssetItem_Id
      UNION ALL
      SELECT AI.[AssetItem_Id] FROM [AssetItems] AI JOIN AI_UnderBase AI2 ON AI2.[AssetItem_Id] = AI.[Parent_AssetItem_Id]
    )    
    SELECT [AssetItem_Id],
           [Match_Index] = CHARINDEX(WORD.[Value_Text] COLLATE Latin1_General_CS_AS, [AssetItem_Lower_Name]),
		   [Match_Percent] = LEN(@Search_Text) / LEN(WORD.[Value_Text])
      INTO #WordMatches
      FROM [AssetItems] AI
           CROSS JOIN dbo.CsvToTable(@Search_Text, ' ') WORD
     WHERE NULLIF(RTRIM(LTRIM(WORD.[Value_Text])), '') IS NOT NULL
       AND (@IncludeDeleted_Indicator = 'Y' OR [Deleted_Date] IS NULL)
       AND (@BasePath_Text IS NULL OR [AssetItem_Id] IN (SELECT [AssetItem_Id] FROM AI_UnderBase))
	   AND [AssetItem_Lower_Name] LIKE ('%' + LOWER(WORD.[Value_Text]) + '%') COLLATE Latin1_General_CS_AS

	-- build a ranked resultset, weighting by count and match index/percent
	SET ROWCOUNT @Row_Count
    SELECT [AssetItem_Id],
	       [Rank] = RANK() OVER (ORDER BY COUNT(*) DESC, MAX(#WordMatches.Match_Percent) DESC, MAX(#WordMatches.Match_Index) ASC)
      INTO #Results
      FROM #WordMatches 
     GROUP BY [AssetItem_Id]
	 ORDER BY [Rank]
	
    ;WITH AI_Flat AS (
      SELECT [Row] = 1, [AssetItem_Id] , [Parent_AssetItem_Id], [AssetItem_Name], [CacheConfiguration_Xml] 
        FROM [AssetItems] WHERE [AssetItem_Id] IN (SELECT [AssetItem_Id] FROM #Results)
      UNION ALL
      SELECT [Row] = [Row] + 1, AI2.[AssetItem_Id], AI.[Parent_AssetItem_Id], AI.[AssetItem_Name], AI.[CacheConfiguration_Xml] 
        FROM [AssetItems] AI INNER JOIN AI_Flat AI2 ON AI2.[Parent_AssetItem_Id] = AI.[AssetItem_Id]
    )
    SELECT AI.*
	
          ,[Canonical_AssetItem_Path] = 
               STUFF( (SELECT '/' + [AssetItem_Name]
                        FROM AI_Flat WHERE AI_Flat.[AssetItem_Id] = AI.[AssetItem_Id]
                       ORDER BY [Row] DESC
                       FOR XML PATH(''), TYPE).value('.', 'nvarchar(max)')
                ,1,1,'')
          
		  ,[Inhereted_CacheConfiguration_Xml] = 
               (SELECT TOP 1 AI_Flat.[CacheConfiguration_Xml]
			      FROM AI_Flat 
				 WHERE AI_Flat.[AssetItem_Id] = [AssetItem_Id]
				   AND AI_Flat.[CacheConfiguration_Xml] IS NOT NULL
				 ORDER BY [Row] ASC)
		  
      FROM [AssetItems] AI 
           JOIN #Results R ON AI.[AssetItem_Id] = R.[AssetItem_Id]
    ORDER BY R.[Rank]

END
GO

GRANT EXECUTE ON [Assets_SearchItems] TO [ProGetUser_Role]