EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Assets_UpdateUserMetadata',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates all user metadata for an asset item.'
GO

IF OBJECT_ID('[Assets_UpdateUserMetadata]') IS NOT NULL DROP PROCEDURE [Assets_UpdateUserMetadata]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Assets_UpdateUserMetadata]
(
	@AssetItem_Id INT,
	@AssetItemMetadataEntry_Table dbo.AssetItemMetadataEntry READONLY,
	@DeleteMissingEntries_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
    BEGIN TRANSACTION

	-- use CTE as filter to prevent full table scan
	;WITH Filtered_AIM
	AS
	(
		SELECT *
		  FROM [AssetItemMetadata]
		 WHERE [AssetItem_Id] = @AssetItem_Id
	)
	MERGE Filtered_AIM AS TARGET
	USING @AssetItemMetadataEntry_Table AS SOURCE
	   ON (
	           TARGET.[AssetItem_Id] = @AssetItem_Id
	       AND TARGET.[Key_Name] = SOURCE.[Key_Name]
		  )

	 WHEN MATCHED AND (TARGET.[Value_Text] <> SOURCE.[Value_Text] OR TARGET.[IncludeInResponseHeader_Boolean] <> SOURCE.[IncludeInResponseHeader_Boolean]) THEN
		UPDATE SET TARGET.[Value_Text] = SOURCE.[Value_Text],
		           TARGET.[IncludeInResponseHeader_Boolean] = SOURCE.[IncludeInResponseHeader_Boolean]

	 WHEN NOT MATCHED BY TARGET THEN
		INSERT ([AssetItem_Id], [Key_Name], [Value_Text], [IncludeInResponseHeader_Boolean])
		VALUES (@AssetItem_Id, SOURCE.[Key_Name], SOURCE.[Value_Text], SOURCE.[IncludeInResponseHeader_Boolean])

	 WHEN NOT MATCHED BY SOURCE AND @DeleteMissingEntries_Indicator = 'Y' THEN
		DELETE;

	DECLARE @Feed_Id INT = (SELECT [Feed_Id] FROM [AssetItems] WHERE [AssetItem_Id] = @AssetItem_Id)

	UPDATE [Feeds]
	   SET [FeedState_Number] = COALESCE([FeedState_Number], 0) + 1
	 WHERE [Feed_Id] = @Feed_Id

    COMMIT
END TRY BEGIN CATCH
    IF XACT_STATE()<>0 ROLLBACK
    EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Assets_UpdateUserMetadata] TO [ProGetUser_Role]
