EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'DockerImages_SetData',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets manifest data associated with an image.'
GO

IF OBJECT_ID('[DockerImages_SetData]') IS NOT NULL DROP PROCEDURE [DockerImages_SetData]
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

CREATE PROCEDURE [DockerImages_SetData]
(
	@DockerImage_Id INT,
	@BlobDigests_Csv VARCHAR(MAX),
	@BlobContentTypes_Csv VARCHAR(MAX),
	@UseCommonBlobStorage_Indicator YNINDICATOR,
	@ContainerConfigBlob_Digest VARCHAR(128),
	@ContainerConfigBlob_ContentType VARCHAR(255),
	@ImageType_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

	BEGIN TRANSACTION

	DECLARE @Feed_Id INT
	SET @Feed_Id = (SELECT [Feed_Id]
	                  FROM [DockerImages] DI
					       INNER JOIN [DockerRepositories] DR
						           ON DR.[DockerRepository_Id] = DI.[DockerRepository_Id]
					 WHERE [DockerImage_Id] = @DockerImage_Id)

	DECLARE @BlobData TABLE
	(
		[DockerBlob_Id] INT NOT NULL,
		[Layer_Sequence] INT NOT NULL,
		[ContentType_Text] VARCHAR(255) NULL
	)

	INSERT INTO @BlobData
	(
		[DockerBlob_Id],
		[Layer_Sequence],
		[ContentType_Text]
	)
	SELECT DB.[DockerBlob_Id],
	       [Layer_Sequence] = D_CSV.[Value_Index],
		   [ContentType_Text] = C_CSV.[Value_Text]
	  FROM dbo.CsvToTable(@BlobDigests_Csv, ',') D_CSV
	       INNER JOIN [DockerBlobs] DB
		           ON DB.[Blob_Digest] = D_CSV.[Value_Text]
				  AND ((@UseCommonBlobStorage_Indicator = 'Y' AND DB.[Feed_Id] IS NULL) OR (@UseCommonBlobStorage_Indicator = 'N' AND DB.[Feed_Id] = @Feed_Id))
		   INNER JOIN dbo.CsvToTable(@BlobContentTypes_Csv, ',') C_CSV
		           ON C_CSV.[Value_Index] = D_CSV.[Value_Index]

	DECLARE @ContainerConfigBlob_Id INT
	SET @ContainerConfigBlob_Id = (SELECT [DockerBlob_Id]
	                                 FROM [DockerBlobs] DB
									WHERE DB.[Blob_Digest] = @ContainerConfigBlob_Digest
									  AND ((@UseCommonBlobStorage_Indicator = 'Y' AND DB.[Feed_Id] IS NULL) OR (@UseCommonBlobStorage_Indicator = 'N' AND DB.[Feed_Id] = @Feed_Id)))

	UPDATE [DockerImages]
	   SET [ContainerConfigBlob_Id] = @ContainerConfigBlob_Id,
	       [ImageType_Code] = @ImageType_Code
	 WHERE [DockerImage_Id] = @DockerImage_Id

	UPDATE DB
	   SET [MediaType_Name] = NULLIF(D.[ContentType_Text], '')
	  FROM [DockerBlobs] DB
	       INNER JOIN @BlobData D
		           ON D.[DockerBlob_Id] = DB.[DockerBlob_Id]
	 WHERE DB.[MediaType_Name] IS NULL OR LEN(DB.[MediaType_Name]) = 0

	IF @ContainerConfigBlob_Id IS NOT NULL
		UPDATE [DockerBlobs]
		   SET [MediaType_Name] = NULLIF(@ContainerConfigBlob_ContentType, '')
		 WHERE [DockerBlob_Id] = @ContainerConfigBlob_Id AND ([MediaType_Name] IS NULL OR LEN([MediaType_Name]) = 0)

	DELETE [DockerImageLayers] WITH (TABLOCKX)
	 WHERE [DockerImage_Id] = @DockerImage_Id

	INSERT INTO [DockerImageLayers]
	(
		[DockerImage_Id],
		[Layer_Sequence],
		[DockerBlob_Id]
	)
	SELECT [DockerImage_Id] = @DockerImage_Id,
	       [Layer_Sequence],
		   [DockerBlob_Id]
	  FROM @BlobData

	COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DockerImages_SetData] TO [ProGetUser_Role]
GO