EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Vulnerabilities_CreateAutoAssessment',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new auto-assessment for a vulnerability if the vulnerability has not already been assessed.'
GO

IF OBJECT_ID('[Vulnerabilities_CreateAutoAssessment]') IS NOT NULL DROP PROCEDURE [Vulnerabilities_CreateAutoAssessment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Vulnerabilities_CreateAutoAssessment]
(
	@Vulnerability_Id INT,
	@AssessmentType_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Vulnerabilities]
	   SET [AssessmentType_Id] = @AssessmentType_Id,
		   [Assessment_Date] = GETUTCDATE(),
		   [AssessedBy_User_Name] = 'SYSTEM'
	 WHERE [Vulnerability_Id] = @Vulnerability_Id
	   AND [Assessment_Date] IS NULL

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Vulnerabilities_CreateAutoAssessment] TO [ProGetUser_Role]
GO